from manim import *
from manim.utils.tex_file_writing import *
from uuid import uuid4
from subprocess import Popen


__all__ = ["RstMobject"]


def create_rst_file(rst: str):
    file_name = f"{uuid4()}.rst"
    with open(file_name, "w") as f:
        f.write(rst)
    return file_name

def convert_rst_file_to_latex(file_name: str):
    Popen(f"rst2latex.py {file_name} {file_name.replace('.rst', '.tex')}").wait()
    return file_name.replace(".rst", ".tex")

def convert_latex_to_tex_str(file_name: str):
    with open(file_name, "r") as f:
        return f.read()

def rst_to_tex_str(rst: str):
    latex_file = convert_rst_file_to_latex(create_rst_file(rst))
    return convert_latex_to_tex_str(latex_file), latex_file

def remove_everything(prefix):
    for file_name in Path(".").glob(f"{prefix}*"):
        file_name.unlink()


class EmptyTexTemplate(TexTemplate):
    def __init__(self):
        super().__init__()

    def _rebuild(self):
        return TexTemplate.default_placeholder_text


class RstMobject(MathTex):
    def __init__(self, rst: str, **kwargs):
        tex_str, latex_file = rst_to_tex_str(rst)
        super().__init__(tex_str, tex_template=EmptyTexTemplate(), **kwargs)
        remove_everything(latex_file.replace(".tex", ""))


from manim import *
from .rst_mobject import RstMobject


__all__ = ["MobjectDescription"]


class MobjectDescription(Group):
    def __init__(
        self,
        mobject: Mobject,
        title: str | None = None,
        description: str | None = None,
        rec_config: dict | None = None,
        **kwargs
    ):
        super().__init__(**kwargs)
        self.mobject = mobject
        self.description = mobject.__doc__ if description is None else description
        examples_idx = self.description.find("Examples")
        if examples_idx != -1:
            self.description = self.description[:examples_idx]
        self.title = mobject.__class__.__name__ or title
        self.title_mobject = Tex(f"\\textbf{{{title}}}", font_size=36)
        self.description_mobject = RstMobject(self.description, font_size=24)
        self.add(self.title_mobject, self.mobject, self.description_mobject)
        self.arrange(DOWN, buff=0.5)
        rec_config = rec_config or {
            "stroke_color": WHITE,
            "stroke_width": 2,
            "fill_color": BLACK,
            "fill_opacity": 0.5,
            "buff": 0.2,
            "corner_radius": 0.2
        }
        self.add(SurroundingRectangle(self, **rec_config))
    
    def set_description(self, description: str):
        self.description = description
        examples_idx = self.description.find("Examples")
        if examples_idx != -1:
            self.description = self.description[:examples_idx]
        self[2].become(RstMobject(self.description, font_size=24))
        return self
    
    def set_title(self, title: str):
        self.title = title
        self[0].become(Tex(f"\\textbf{{{title}}}", font_size=36))
        return self
    
    def set_mobject(self, mobject: Mobject, description: str | None = None, title: str | None = None):
        self[1].become(mobject)
        self.set_description(mobject.__doc__ if description is None else description)
        self.set_title(mobject.__class__.__name__ or title)
        return self



from manim import *
from .rst_mobject import RstMobject


__all__ = ["AnimationDescription"]


class AnimationDescription(Group):
    def __init__(
        self,
        mobject_to_animate: Mobject,
        animation: Animation,
        title: str | None = None,
        description: str | None = None,
        rec_config: dict | None = None
    ):
        super().__init__()
        self.mobject_to_animate = mobject_to_animate
        self.animation = animation
        self.title = animation.__class__.__name__ or title
        self.description = description or animation.__doc__
        examples_idx = self.description.find("Examples")
        if examples_idx != -1:
            self.description = self.description[:examples_idx]
        self.rec_config = rec_config or {
            "stroke_color": WHITE,
            "stroke_width": 2,
            "fill_color": BLACK,
            "fill_opacity": 0.5,
            "buff": 0.2,
            "corner_radius": 0.2
        }
        self.title_mobject = Tex(f"\\textbf{{{self.title}}}", font_size=36)
        self.description_mobject = RstMobject(self.description, font_size=24)
        self.add(self.title_mobject, self.mobject_to_animate, self.description_mobject)
        self.arrange(DOWN, buff=0.5)
        self.add(SurroundingRectangle(self, **self.rec_config))
    
    def set_description(self, description: str):
        self.description = description
        examples_idx = self.description.find("Examples")
        if examples_idx != -1:
            self.description = self.description[:examples_idx]
        self[2].become(RstMobject(self.description, font_size=24))
        return self
    
    def set_title(self, title: str):
        self.title = title
        self[0].become(Tex(f"\\textbf{{{title}}}", font_size=36))
        return self
    
    def set_mobject_and_animation(
        self,
        mobject_to_animate: Mobject,
        animation: Animation,
        description: str | None = None,
        title: str | None = None
    ):
        self[1].become(mobject_to_animate)
        self.set_description(description or animation.__doc__)
        self.set_title(animation.__class__.__name__ or title)
        self.animation = animation
        return self
    
    def set_animation(self, animation: Animation, description: str | None = None, title: str | None = None):
        self.set_mobject_and_animation(self.mobject_to_animate, animation, description, title)
        return self
    
    def set_mobject(self, mobject: Mobject, description: str | None = None, title: str | None = None):
        self.set_mobject_and_animation(mobject, self.animation, description, title)
        return self
    
    def play_animation(self, scene: Scene, run_time: float | None = 1, **kwargs):
        if run_time is not None:
            kwargs["run_time"] = run_time
        scene.play(self.animation, **kwargs)
        return self
